//*******************************************************************************
//	Copyright (C) 2013 John Chapman

//	This software is distributed freely under the terms of the MIT License.
//	See "license.txt" or "http://copyfree.org/licenses/mit/license.txt".
//*******************************************************************************/

// Depth vs
void CMB_Depth_Vs
(
    in float4 inPos : POSITION
    ,out float4 pos : POSITION
    ,out float2 vDepth : TEXCOORD0
    ,uniform float4x4 worldViewProj
)
{
    pos = mul(worldViewProj, inPos);
	vDepth.x = pos.z;
	vDepth.y = pos.w;
}

// Depth ps
void CMB_Depth_Ps
(
    in float2 vDepth : TEXCOORD0
    ,out float4 fResult : COLOR
)
{
	float finalDepth = vDepth.x / vDepth.y;   
	fResult = float4(finalDepth,finalDepth,finalDepth,1.0);
}

// MotionBlur vs
void CMB_Vs
(
    in float4 ip : POSITION
    ,out float4 pos : POSITION
    ,out float2 vTexcoord : TEXCOORD0
    ,uniform float4x4 worldViewProj
)
{
    pos = mul(worldViewProj, ip);
    float2 inPos = sign(ip.xy);
    vTexcoord = (float2(inPos.x, -inPos.y) + 1.0f) * 0.5f;
}

// MotionBlur ps
void CMB_Ps
(
    float2 vTexcoord : TEXCOORD0
    ,out float4 fResult : COLOR
    ,uniform float4x4 uInverseViewProjectionMatrix
	,uniform float4x4 uPrevViewProjectionMatrix
	,uniform float4 uInputTextureSize
	,uniform sampler2D uInputTex:register(s0)
	,uniform sampler2D uDepthTex:register(s1)
)
{
	float depth = tex2D(uDepthTex, vTexcoord).r;
	float4 cpos = float4(vTexcoord * 2.0 - 1.0, depth, 1.0);
	cpos = mul(uInverseViewProjectionMatrix, cpos);
	float4 ppos = mul(uPrevViewProjectionMatrix,cpos);
	ppos.xyz /= ppos.w;
	ppos.xy = ppos.xy * 0.5 + 0.5;
	
	float2 velocity = (ppos.xy - vTexcoord);
	
	float2 texelSize = 1.0 / uInputTextureSize.xy;
	
	float tspeed = length(velocity / texelSize); // speed in pixels/second
	float nSamples = floor(clamp(tspeed, 1, 64));

	float4 result = tex2D(uInputTex, vTexcoord);
	int loop = int(nSamples) % 65;
	for (int i = 1; i < loop; ++i) {
		float2 offset = vTexcoord + velocity * (float(i) / (nSamples - 1) - 0.5);
		result += tex2D(uInputTex, offset);
	}
	fResult = result / nSamples;
}