/*******************************************************************************
	Copyright (C) 2013 John Chapman

	This software is distributed freely under the terms of the MIT License.
	See "license.txt" or "http://copyfree.org/licenses/mit/license.txt".
*******************************************************************************/
#version 120

uniform mat4 uInverseViewProjectionMatrix;
uniform mat4 uPrevViewProjectionMatrix;
uniform vec4 uInputTextureSize;

uniform sampler2D uInputTex;
uniform sampler2D uDepthTex;

varying vec2 vTexcoord;

void main() {
	float depth = texture2D(uDepthTex, vTexcoord).r;
	vec4 cpos = vec4(vTexcoord * 2.0 - 1.0, depth, 1.0);
	cpos = uInverseViewProjectionMatrix * cpos;
	vec4 ppos = uPrevViewProjectionMatrix * cpos;
	ppos.xyz /= ppos.w;
	ppos.xy = ppos.xy * 0.5 + 0.5;
	
	vec2 velocity = (ppos.xy - vTexcoord);
	
	vec2 texelSize = 1.0 / uInputTextureSize.xy;
	
	float tspeed = length(velocity / texelSize); // speed in pixels/second
	float nSamples = floor(clamp(tspeed, 1, 64));

	vec4 result = texture2D(uInputTex, vTexcoord);
	for (int i = 1; i < int(nSamples); ++i) {
		vec2 offset = vTexcoord + velocity * (float(i) / (nSamples - 1) - 0.5);
		result += texture2D(uInputTex, offset);
	}
	gl_FragColor = result / nSamples;
}
